#!/usr/bin/env python
#Author: Mark Feineigle
#Create Date: 2018/05/02
#Last Modify: 2018/05/02
# deeper learning insights machine learning page 606

from sklearn import cross_validation
from sklearn.tree import DecisionTreeClassifier
from sklearn import svm
from sklearn.linear_model import LogisticRegression
from sklearn.datasets import samples_generator
from sklearn.preprocessing import LabelEncoder
from sklearn.preprocessing import StandardScaler
from sklearn.cross_validation import cross_val_score
from sklearn.pipeline import Pipeline

X, y = samples_generator.make_classification(n_samples=1000,
                                             n_informative=5,
                                             n_redundant=0,
                                             random_state=42)
le = LabelEncoder()
y = le.fit_transform(y)
Xtrain, Xtest, ytrain, ytest = cross_validation.train_test_split(X, y,
                                                                 test_size=0.5,
                                                                 random_state=1)

clf1 = DecisionTreeClassifier(max_depth=2,criterion='gini').fit(Xtrain,ytrain)
clf2 = svm.SVC(kernel='linear', probability=True, random_state=0).fit(Xtrain,ytrain)
clf3 = LogisticRegression(penalty='l2', C=0.001).fit(Xtrain,ytrain)
pipe1 = Pipeline([['sc',StandardScaler()],['mod',clf1]])

mod_labels = ['Decision Tree', 'SVM', 'Logistic Regression']
print('10 fold cross validation: \n')
for mod, label in zip([pipe1, clf2, clf3], mod_labels):
  #print(label)
  auc_scores = cross_val_score(estimator=mod,
                               X=Xtrain,
                               y=ytrain,
                               cv=10, 
                               scoring='roc_auc')
  p_scores = cross_val_score(estimator=mod,
                             X=Xtrain,
                             y=ytrain,
                             cv=10,
                             scoring='precision_macro')
  r_scores = cross_val_score(estimator=mod,
                             X=Xtrain,
                             y=ytrain,
                             cv=10,
                             scoring='recall_macro')
  f_scores= cross_val_score(estimator=mod,
                            X=Xtrain,
                            y=ytrain,
                            cv=10,
                            scoring='f1_macro')
  print(label)
  print("auc scores %2f +/- %2f " % (auc_scores.mean(), auc_scores.std()))
  print("precision %2f +/- %2f " % (p_scores.mean(), p_scores.std()))
  print("recall %2f +/- %2f ]" % (r_scores.mean(), r_scores.std()))
  print("f scores %2f +/- %2f " % (f_scores.mean(), f_scores.std()))
  #NOTE this output does not match the book


# LASSO CROSS VALIDATION
import numpy as np
from sklearn import cross_validation, datasets, linear_model
X,y=datasets.make_blobs(n_samples=80,centers=2, random_state=0, cluster_std=2)
alphas = np.logspace(-4, -.5, 30)
lasso_cv = linear_model.LassoCV(alphas=alphas)
k_fold = cross_validation.KFold(len(X), 5)
alphas = np.logspace(-4, -.5, 30)
for k, (train, test) in enumerate(k_fold):
  lasso_cv.fit(X[train], y[train])
  print("[fold {0}] alpha: {1:.5f}, score: {2:.5f}".format(k,
                                                           lasso_cv.alpha_,
                                                           lasso_cv.score(X[test],
                                                                          y[test])))
  #NOTE this output does not match the book


## cross_validation.permutation_test_score
import numpy as np
from sklearn import linear_model
from sklearn.cross_validation import StratifiedKFold, permutation_test_score
from sklearn import datasets
X,y=datasets.make_classification(n_samples=100, n_features=5)
n_classes = np.unique(y).size
cls=linear_model.LogisticRegression()
cv = StratifiedKFold(y, 2)
score, permutation_scores, pvalue = permutation_test_score(cls,
                                                           X,
                                                           y,
                                                           scoring="f1",
                                                           cv=cv,
                                                           n_permutations=10,
                                                           n_jobs=1)
print("Classification score %s (pvalue : %s)" % (score, pvalue))
print("Permutation scores %s" % (permutation_scores))
#NOTE this output does not match the book, NOTE no random state?
